; Jump to start
    ; set startstkp = sp for exit()
    ld  hl, 0
    add hl, sp
    ex  de,hl
    ld  hl, startstkp
    ld  (hl), d
    inc  hl
    ld  (hl), e
    JP main
startstkp:
.dw 0000h;
;	Run time start off for Small C.
;
;*****************************************************
;*						     *
;*	runtime	library	for small C compiler	     *
;*						     *
;*	c.s - runtime routine for basic C code	     *
;*						     *
;*		Ron Cain			     *
;*						     *
;*****************************************************
;
; fetch char from (HL) and sign extend into HL
_gchar: ld      a,(hl)	;7 cycles
_sxt:   ld      l,a	;4
	rlca		;4
	sbc	a,a	;? cycles
	ld	h,a	;4
	ret		;10
		;tot: 29/22+? cycles
;Searched for _gchar & _sxt:
;TI83: nothing found

;Although the one below seems to be alright. I have the
;idea that the original version is better in some cases.
;_gchar: ld	a,(hl)	;7 clk	;1 byte
;_sxt:	ld	l,a	;4	;1
;	ld	h,0	;7	;2
;	ret		;10
;		;tot: 21/28 clk


; Optimizer inline replaces these commands for speed
#IFNDEF OPTIMIZE
; fetch int from (HL)
_gint:  ld      a,(hl)	;7 cycles
	inc	hl	;6
	ld	h,(hl)	;7
	ld	l,a	;4
	ret		;10
		;tot: 34 cycles
;Searched for _gint: 
;TI83: $0033 ($4000 is probably a jump to there...)

; store char from HL into (DE)
_pchar: ld      a,l	;4
	ld	(de),a	;13
	ret		;10
		;tot: 27 cycles
;Searched for _pchar:
;TI83: nothing found
#ENDIF

; store int from HL into (DE)
_pint:  ld      a,l	;4
	ld	(de),a	;13
	inc	de	;6
        ld      a,h	;4
        ld      (de),a	;13
	ret		;10
		;tot: 49 cycles
;Searched for _pint:
;TI83: nothing found

; "or" HL and DE into HL
_or:    ld      a,l	;4
	or	e	;4
	ld	l,a	;4
	ld	a,h	;4
	or	d	;4
	ld	h,a	;4
	ret		;10
		;tot: 38 cycles

; "xor" HL and DE into HL
_xor:   ld      a,l	;4
	xor	e	;4
	ld	l,a	;4
	ld	a,h	;4
	xor	d	;4
	ld	h,a	;4
	ret		;10
		;tot: 38 cycles

; "and" HL and DE into HL
_and:   ld      a,l	;4
	and	e	;4
	ld	l,a	;4
	ld	a,h	;4
	and	d	;4
	ld	h,a	;4
	ret		;10
		;tot: 38 cycles
;
;______logical operations: HL set to 0 (false) or 1 (true)
;
; DE == HL
_eq:    call    _cmp	;17
        ret     z	;11/5
	dec	hl	;6
	ret		;10
		;tot: 44/38 cycles
; DE != HL
_ne:    call    _cmp	;17
	ret	nz	;11/5
	dec	hl	;6
	ret		;10
		;tot: 44/38 cycles
; DE > HL [signed]
_gt:    ex      de,hl	;4
;	call    _cmp	;17
;	ret	c	;11/5
;	dec	hl	;6
;	ret		;10
;NEXT ROUTINE IS THE SAME...

; DE < HL [signed]
_lt:    call    _cmp	;17
	ret	c	;11/5
	dec	hl	;6
	ret		;10
		;tot: 44/38 clk

; DE <= HL [signed]
_le:    call    _cmp	;17
	ret	z	;11/5
	ret	c	;11/5
	dec	hl	;6
	ret		;10
		;tot: 55/49/43 cycles
; DE >= HL [signed]
_ge:    call    _cmp	;17
	ret	nc	;11/5
	dec	hl	;6
	ret		;10
		;tot: 44/38 clk

; DE >= HL [unsigned]
_uge:   call    _ucmp	;17
	ret	nc	;11/5
	dec	hl	;6
	ret		;10
		;tot: 44/38 clk

; DE > HL [unsigned]
_ugt:   ex      de,hl	;4
;	call    _ucmp	;17
;	ret	c	;11/5
;	dec	hl	;6
;	ret		;10
;NEXT ROUTINE IS THE SAME...

; DE < HL [unsigned]
_ult:   call    _ucmp	;17
	ret	c	;11/5
	dec	hl	;6
	ret		;10
		;tot: 44/38 clk

; DE <= HL [unsigned]
_ule:   call    _ucmp	;17
	ret	z	;11/5
	ret	c	;11/5
	dec	hl	;6
	ret		;10
		;tot:
; signed compare of DE and HL
;   carry is sign of difference [set => DE < HL]
;   zero is zero/non-zero
#IFDEF OLD
_cmp:   ld      a,e	;4
	sub	l	;4
	ld	e,a	;4
	ld	a,d	;4
	sbc	a,h	;4
        ld      hl,1	;10	;preset true
	jp      m,_cmp1	;10/1	;'jr' doesn't understand 'm'...
        or      e	;4	;resets carry
	ret		;10
_cmp1:	or	e	;4
	scf		;4	;set carryflag
	ret		;10
		;tot:
#ELSE
_cmp:   ld      a,e	;4
	sub	l	;4
	ld	e,a	;4
	ld	a,d	;4
	sbc	a,h	;4
        ld      hl,1	;10	;preset true
	scf		;4	;set carryflag
	ret	m	;	;IF signflag = 1 THEN return ELSE ...
_cmp1:	or	e	;4	;reset carryflag
	ret		;10
		;tot:
#ENDIF

; unsigned compare of DE and HL
;   carry is sign of difference [set => DE < HL]
;   zero is zero/non-zero
_ucmp:  ld      a,d	;4
	cp	h	;4
        jr      nz,_ucmp1 ;10/1
	ld	a,e	;4
	cp	l	;4
_ucmp1: ld      hl,1 	;10	;preset true
	ret		;10
		;tot:
; shift DE right arithmetically by HL, move to HL
_asr:   ex      de,hl	;
_asr1:  dec     e	;
	ret	m	;11/5
	ld	a,h	;4
	rla		;4	;rotate left acc.
	ld	a,h	;4
	rra		;4	;rotate right acc.
	ld	h,a	;4
	ld	a,l	;4
	rra		;4
	ld	l,a	;4
        jr      _asr1	;12
		;tot:
; shift DE left arithmetically by HL, move to HL
_asl:   ex      de,hl	;4
_asl1:  dec     e	;4
	ret	m	;11/5
	add	hl,hl	;11
        jr      _asl1	;12
		;tot:
; HL = DE - HL
_sub:   ld      a,e	;4
	sub	l	;4
	ld	l,a	;4
	ld	a,d	;4
	sbc	a,h	;7?	;not in my op-code table (only SBC A,number)
	ld	h,a	;4
	ret		;10
		;tot:
; HL = -HL
_neg:   call    _com	;17
	inc	hl	;6
	ret		;10
		;tot: 33
; HL = ~HL
_com:   ld      a,h	;4
	cpl		;4	;complement (same as "OR $FF")
	ld	h,a	;4
	ld	a,l	;4
	cpl		;4	;complement
	ld	l,a	;4
	ret		;10
		;tot:
;Searched for _com (also with 'cpl' changed to 'or $ff'):
;TI83: nothing found

; HL = !!HL
_bool:	call	_lneg	;17
	;jr	_lneg	;12	;will 'fall' to next routine. (2 bytes saved)
		;tot: 17 (dh)
; HL = !HL
_lneg:  ld      a,h	;4
	or	l	;4
        jr      z,_lneg1;12/7
        ld      hl,0	;10
	ret		;10
_lneg1: inc     hl	;6
	ret		;10
		;tot:
;
; HL = DE * HL [signed]
_mul:   ld      b,h	;4
	ld	c,l	;4
        ld      hl,0	;10
_mul1:  ld      a,c	;4
	rrca		;4	;Rotate Right Cir.Acc.
        jr      nc,_mul2;
	add	hl,de	;
_mul2:  xor     a	;
	ld	a,b	;4
	rra		;
	ld	b,a	;4
	ld	a,c	;4
	rra		;
	ld	c,a	;4
	or	b	;
	ret	z	;11/5
	xor	a	;4	;gets overwritten ? (OPTIMIZE)
	ld	a,e	;4
	rla		;
	ld	e,a	;4
	ld	a,d	;4
	rla		;
	ld	d,a	;4
	or	e	;
	ret	z	;11/5
        jr      _mul1	;
		;tot:
; HL = DE / HL, DE = DE % HL
_div:   ld      b,h	;4
	ld	c,l	;4
	ld	a,d	;4
	xor	b	;4
	push	af	;11
	ld	a,d	;4
	or	a	;4
        call    m,_deneg;
	ld	a,b	;4
	or	a	;
        call    m,_bcneg;
        ld      a,16	;7
	push	af	;11
	ex	de,hl	;
        ld      de,0	;10
_div1:  add     hl,hl	;
        call    _rdel	;17
        jr      z,_div2	;
        call    _cmpbd	;17
        jp      m,_div2	;
	ld	a,l	;4
        or      1	;7
	ld	l,a	;4
	ld	a,e	;4
	sub	c	;
	ld	e,a	;4
	ld	a,d	;4
	sbc	a,b	;
	ld	d,a	;4
_div2:  pop     af	;10
	dec	a	;
        jr      z,_div3	;
	push	af	;11
        jr      _div1	;
_div3:  pop     af	;10
	ret	p	;11/5
        call    _deneg	;17
	ex	de,hl	;
        call    _deneg	;17
	ex	de,hl	;
	ret		;10
		;tot:
; {DE = -DE}
_deneg: ld      a,d	;4
	cpl		;	;complement
	ld	d,a	;4
	ld	a,e	;4
	cpl		;	;complement
	ld	e,a	;4
	inc	de	;6
	ret		;10
		;tot:
; {BC = -BC}
_bcneg: ld      a,b	;4
	cpl		;
	ld	b,a	;4
	ld	a,c	;4
	cpl		;
	ld	c,a	;4
	inc	bc	;6
	ret		;10
		;tot:
; {DE <r<r 1}
_rdel:  ld      a,e	;4
	rla		;
	ld	e,a	;4
	ld	a,d	;4
	rla		;4
	ld	d,a	;4
	or	e	;
	ret		;10
		;tot:
; {BC : DE}
_cmpbd: ld      a,e	;4
	sub	c	;
	ld	a,d	;4
	sbc	a,b	;
	ret		;10
		;tot:
; case jump
_case:  ex      de,hl	;	;switch value to DE
	pop	hl	;10	;get table address
_case1: call    _case4	;17	;get case value
	ld	a,e	;4
        cp      c	;	;equal to switch value _
        jr      nz,_case2;	;no
	ld	a,d	;4
        cp      b	;	;equal to switch value _
        jr      nz,_case2;	;no
        call    _case4	;	;get case label
        jr      z,_case3;	;end of table, go to default
	push	bc	;
	ret		;	;case jump
_case2: call    _case4	;	;get case label
        jr      nz,_case1;	;next case
_case3: dec     hl	;
	dec	hl	;
	dec	hl	;
	ld	d,(hl)	;
	dec	hl	;
	ld	e,(hl)	;
	ex	de,hl	;
	jp	(hl)	;	;default jump
_case4: ld      c,(hl)	;
	inc	hl	;
	ld	b,(hl)	;
	inc	hl	;
	ld	a,c	;4
	or	b	;
	ret		;10
		;tot:

#IFDEF OPTIMIZE
_zbool:	call	_lneg	;17
	call	_lneg	;12
	ld	a,h	;4
	or	l	;7
	ret
		;tot:

_zeq:	call    _cmp	;17
        jr	z,_zeq2	;
	dec	hl	;6
_zeq2:	ld	a,h	;4
	or	l	;7
	ret		;10
		;tot:

_zgt:	ex      de,hl	;4
;	call    _cmp	;17
;	jr	c,_zgt2	;
;	dec	hl	;6
;_zgt2:	ld	a,h	;4
;	or	l	;7
;	ret		;10
;NEXT ROUTINE IS THE SAME...

_zlt:	call    _cmp	;17
	jr	c,_zlt2	;
	dec	hl	;6
_zlt2:	ld	a,h	;4
	or	l	;7
	ret		;10


_zneg:	call    _com	;17
	inc	hl	;6
	ld	a,h	;4
	or	l	;7
	ret		;10
#ENDIF
