#ifndef __FADEIN_C
#define __FADEIN_C

/* Fade in, from contrast 0 till (curcont)

   Small-C library, by Henk Poley (hpoley@dds.nl)
   (---no www addres---)

   Synopsis:
      fade_in()

   Description:
      Fade in from contrast 0 till (curcont)

   Example:
      Fade_in();

   Output:
      The screen will turn white, and slowly return to the contrast it
      was before.

   Notes:
      o You can use this in intro screens. (dh (a bit))

      o It can be optimized... (and then to know it is already
        highly optimized in comparsion to the the on I found...)

      o Is the timing to slow? Do you have to wait to long till the
        routine is ended?

*/

#include <delay.c>

fade_in()
{
#asm
#DEFINE	curcont		8008h	; Where the TIOS stores the contrast

	ld	a,(curcont)
	add	a,216		; add 216 so the LCD will understand it
	ld	b,a		; save this value to b
	ld	a,216		; we want to set the contrast to 0
	EX	(sp),hl		; Delay routine for LCD
	EX	(sp),hl		;
				; We don't need it in the next routine
				;  because the routine itself is long
				;  anough
FADE:
	out	(LCDINSTPORT),a ; set the contrast
				;
	ld	hl,20		; 20 * 0,2 sec. delay.
	ld	c,a		; workaround for 'dirty' delay
	call	__fastdelay	;  routine, it affects
	ld	a,c		;  the a register.
				;
	inc	a		; add 1 to contrast
	cp	b		; is it already at the right contrast?
	jr	nz,FADE		; no, again
#endasm
}
#endif