#ifndef _LINKSUPPORT_
#define _LINKSUPPORT_

/* Link-routine for the TI-83

   Small-C library, by Henk Poley (hpoley@dds.nl)
   (---no www addres---)

   Synopsis:
      getbyte()
      sendbyte(char s)

   Description:
      This function is used to get or to send a byte thrue the
      linkport. It will wait till the other calc 'answers the phone'.

   Example:
      chat = getbyte();
      sendbyte(15);

   Notes:
      o The sendbyte() routine calls to some lowlevel stuff in the
        getbyte() routine. I hope they don't bite each other ;)

      o I removed the lines that Florent Dhordain 'remarked' (an ';' in 
        front of it). I know that it will be more difficulty to port
        this routine to another calc. Especially the TI-85, where this
        routine originaly came from.
        But most of the things are really lowlevel stuff, that won't
        work directly on another calc. So you better look for a routine
        that is made for your particular calc, than porting this one.
   
      o I did not test them, I just copied them from "Ztetris1.1 for
        Ion". And I 'beautified' them, to make them more readable
        (I hope).

	(The game itself didn't work in linkplay mode. I hope this is
        because of the game, not the link routine...)

        At the moment it won't produce any errors anymore in TASM. But 
        if it works? In a test my VirtualTi did not crash but, waited
        'till doomsday', because 'no one answered the call'. I know,
        with a VTi you should be able to run linkgames against two
        running instances of the program. I only have a Pentium 100,
        that means the Z80-emulation runs at 70-80%... (So I don't
        think that opening two VTi's to play a link game is likely
        to work)

	And another reason. I should first have to make a program that
        uses the link routine, in a way it works (hmmmm).


-> A note that came with this linkroutine: 

;(The 83+ link port works very simply (the rest should be like this))
;(Line 0 is bit 0)
;(Line 1 is bit 1)
;(It works the same for input and output)
;
; SOME VERY NICE LINK ROUTINES MADE BY PASCAL BOURON
;
; I've made some modifications so it fits ZTetris
;
; Ported to TI83 and highly optimized by Florent Dhordain 06/04/98 MMDDYY
;
*/

getbyte()
{
#asm
	ld	l,0		; l used as lcounter
	ld	b,8		; 8 bits to receive
				; (reset the port to high, both lines)
	ld	a,0
	out	(0),a

rb_w_Start:
	in	a,(0)
				; wait one line becomes low
	and	3		; (check if either line is low)
	cp	3
	jr 	nz,rb_get_bit
	call	Test_ON
	jr	rb_w_Start	; Hmpf, it wasn't low, jump back

rb_get_bit:
	ld	l,0   		; reset lcounter
				; is it a 0 ? (is line1 low?)
	cp	2
	jr	z,rb_receive_zero	; Yeah !
	ld	a,1
	out	(0),a		; Ack for a '1'
	rr	c		; put the 1 (cf has been set by cp)

rb_waitstop1:
	call	Test_ON
	in	a, (0)
				; wait 'til Ack is recognized
	and	2		; (line1 pulled low?)
	jr	z, rb_waitstop1
	jr	rb_stopok

rb_receive_zero:
				; Ack for a '0'
	ld	a,2
	out	(0),a		; (pull line1 low)
	rr	c		; put the 0 (cf has been cleared by cp)

rb_waitStop0:
	call	Test_ON
	in	a,(0)
				; wait 'til Ack is recognized
	and	1		; (is line0 low (the ack))
	jr	z,rb_waitStop0

rb_stopok:              	; here, one line is low cause of us(recv)
				; the sender calc has already set lines to Hi
	ld	a,0		; (reset port to lines high)
	out	(0),a
	djnz	rb_w_Start	; Have we got the whole byte? No?, jump!

				; Yes, we got it. Return to Small-C program.
	ld	l,c   		; Return the value of 'c'
	ld	h,0		; Set 'h' to 0
	ret			; Return (dh...)

Test_ON:
	dec	l
	ret	nz		; exit only if cnt = 0
	pop	hl		; Back to the place you were before.
				;  Gotta love it!
	ld	a,0		; (reset port to lines high)
	out	(0),a
	xor	a
	ret
#endasm
}

sendbyte(s) char s;
{
#asm
	ld	hl,2		; 2 is the offset of the last parameter
	add	hl,sp		; Add the stack to the offset
	ld	a,(hl)		; set a to the high byte of the pointer location

	ld	c,a		; byte to send (using bit rotates into 
				;  carry to find bit to send)
	ld	d,a		; Save it
	ld	hl,0
	ld	(lcounter),hl
	ld	b,8
	ld	a,0		; (reset port to all high)
	out	(0),a

w_setport3:
	in	a,(0)
				; wait both lines are low
	and	3		; (are either line low?)
	cp	3
	jr	z, calc_bit
	call	SendTest_ON
	jr	w_setport3

calc_bit:
	rr	c		; (is the next bit to send a 1 or 0?)
	ld	a,2
	jr	c,send_one	; (if its a 1, goto send_one)

send_zero:
	dec	a

send_one:
	out	(0),A

wait_setport:
	call	SendTest_ON
	in a,(0)
				; wait both become low
	and	3		; (are both lines low?)
	jr	nz,wait_setport
				; put them back to hi
	ld 	a,0		; (reset both lines to high)
	out	(0),A
	djnz	w_setport3
	ld	a,d		; Restore byte
	ret

SendTest_ON:
	ld	a,(hsflag)
	or	a
	jr	z,LongWait
	call	_getK
	cp	$09		; If this is "kClear", exit !
	ret	nz
	pop	hl
	pop	hl
	ret

LongWait:
	ld	hl,(lcounter)
	inc	hl
	ld	(lcounter),hl
	bit	6,h		; Has the counter reached $4000
	ret	z		; no, continue
	pop	hl
	ret

lcounter:			; I need some place to write to, don't I?
	nop			;
hsflag:				;
	nop			;

#endasm
}

/* another link routine,
   I found it in the program "Tron" by a (French) programmer
   Anthony Baire

   I think it only returns the bit that is send / received. The only
   way to know that is by knowing what _IO_EXEC does do. Could somebody
   tell me / make the routine above better?

	Henk Poley

;;;; Fonction getbyte	--> A=0 ou 1
getbyte:
	ld 	a,22d
	ld	(ASM_IND_CALL),a
	push	de
	call	_IO_EXEC
	pop	de

	ret
;;;; Fonction sendbit	<-- A=0 ou 1
sendbyte:
	push	af
	ld 	a,11d
	ld	(ASM_IND_CALL),a
	pop	af
	push	de
	call	_IO_EXEC
	pop	de

	ret
*/
#endif	/* _LINKSUPPORT_ loaded */