#ifndef __LINE_C
#define __LINE_C

/*
   Small c library, by Chris Dannemiller (dannemiller@hotmail.com)
   http://www.geocities.com/SiliconValley/Vista/1143

   Synopsis:
      line(x1,y1,x2,y2,c)

   Description:
      This fuction draws a line from (x1,y1)-(x2,y2). Most of the code
      was taken for Small C for the TI-85 Allthough I fixed a bug in
      this code. You also have to specify the color mode you will be
      drawing in.

   Example:
      line(5,5,20,30,PT_ON)
      line(6,7,32,22,PT_CHG)

   Notes:
      o See ti.h for description of the PT_xx macros

      o Maybe an idea to add some code to check if there are really
        5 parameters given (a=5)
        	Henk Poley

      o 14-8-2000: Added program writeback to this routine so it is
        smaller (YES, PROGRAM WRITEBACK!!!)
        PS. I hope this works on all Ti-eighteens...
                Henk Poley
*/

#include <pixel.c>

line (x1,y1,x2,y2,c) char x1,y1,x2,y2,c;
{
#asm
	; Grab arguments off stack
	ld	hl,2
	add	hl,sp
	ld	a,(hl)
	ld	(_color_lbl+1),A	;for program writeback
	inc	hl
	inc	hl
	ld	d,(hl)
	inc	hl
	inc	hl
	ld	e,(hl)
	inc	hl
	inc	hl
	ld	c,(hl)
	inc	hl
	inc	hl
	ld	l,(hl)
	ld	h,c

	ld	a,d
	sub	h
        jr      nc,_lndxpos     ;delta_x > 0
_lndxneg:
	neg
_lndxpos:
        ld      b,a             ;b <- |delta_x|
        ld      a,e            
        sub     l               
        jr      nc,_lndypos     ;delta_y > 0
_lndyneg:
	neg
_lndypos:
        sub     b               ;|delta_y|
        push    af
        jr      c,_lndx         ;|delta_x| > |delta_y|
_lndy:
        ld      a,h             ;if |delta_x| < |delta_y| then
        ld      h,l             ;then values x and y are swapped
        ld      l,a             ;so the loop will always be performed on the
        ld      a,d             ;x value. A flag must be set to
        ld      d,e             ;remind that data must be drawn (y,x)
        ld      e,a             ;instead of (x,y)
_lndx:
        ld      a,d
        sub     h
        jr      nc,_lntdy       ;x1 < x2
_lntdx:
        ex      de,hl
_lntdy:
	inc	d
        ld      a,e
        sub     l
        ld      a,$01
        jr      nc,_lnsta
	neg			;y1 > y2 : in _lncase2 the 'y' variable
_lnsta:
        ld      (_lnway_lbl+1),a        ;for program writeback
_lninitl:
        ld      b,h
        ld      c,l
        ld      a,e
        sub     l
        jr      nc,_lneinit
        ld      a,l
        ld      l,e
        ld      e,a
_lneinit:
	ld      a,e
        sub     l
	rla
        ld      l,a             ;value to add in _lncase1 (d < 0)
        add     a,h
        sub     d
        ld      e,a             ;'d' variable is initialised
        add     a,h
        sub     d
        ld      h,a             ;value to add in _lncase2 (d >= 0)
_lnloop:
        ld      a,b
        sub     d
        jr      nc,_lnend       ;the line is completely drawn.
        pop     af
        bit     7,a
        push    af
        push    af
        push    bc
        jr      z,_lndpt
        ld      a,b
        ld      b,c
        ld      c,a
_lndpt:
	push	de
	push	hl
_color_lbl:
        ld      a,00000000b     ;program writeback...
	ld	d,c
	ld	e,b
	call	_drawpix
	pop	hl
	pop	de
        pop     bc
        pop     af
        bit     7,e
        jr      nz,_lncase1
_lncase2:			;d >= 0 
        ld      a,e
        add     a,h
        ld      e,a
_lnway_lbl:
        ld      a,00000000b     ;program writeback...
        add     a,c
        ld      c,a
	jr	_lneloop
_lncase1:			;d < 0
        ld      a,e
        add     a,l
        ld      e,a
_lneloop:
        inc     b
	jr	_lnloop
_lnend:
	inc	sp		;MUST NOT BE REMOVED
	inc	sp
#endasm
}
#endif 
