/*
   TI-83/86 Lights out example

   Does anybody know why the first 25 levels aren't used???

   Compiled size
   - TI83:
    o No-opt: 6639 bytes (old value..)
    o opt: 5921
           5903 (optimized 'X' when you made to much moves)
           5806 (optimized releasebutton()/pushbutton(), they used the
                 same routine for stuffing out the old button before 
                 drawing the new one)
           5767 (new Rules optimizations!)

                 ----Henk Poley---
 */

/* Here can you set the first level that will be used */
#define FIRSTLEVEL 1


#include <ti.h>
#include <line.c>
#include <box.c>
#include <delay.c>
#include <fastcopy.h> /* fastcopy now takes automaticly care of type of calculator */

#name "Lights Out 8X"

char board[25];
char xpos, ypos;
char moves;
char minmoves;
extern char level;
extern char levels[];

/* Size of the boxs */
#define xsize 13
#define ysize 12

#define setboard(x,y,v) board[(y*5)+x] = v
#define getboard(x,y)   board[(y*5)+x]

main() {
 char inloop, key, cmate, x, z, a, b;

 /* Erase the contents of the screen */
 indicatoroff;
 clrlcd;

 level = FIRSTLEVEL;

 /* Setup a intro Screen */
 setxy(0,0);

 /* display intro screen */
 /* slightly different for each calc */

#ifdef __TI82__
 puts("\n\
   Lights-Out\n\
A Ti82cc Example\n\
\n\
   By: Chris\n\
  Dannemiller");
#endif

#ifdef __TI83__
 puts("\n\
   Lights-Out\n\
A Ti83cc Example\n\
\n\
   By: Chris\n\
  Dannemiller");
#endif

#ifdef __TI86__
 puts("\n\
     Lights-Out\n\
  A Ti-86cc Example\n\
\n\
     By: Chris\n\
    Dannemiller");
#endif

 /* Wait for user key press */
 getkey();

 /* Clear screen contents */
 clrlcd;

 /* Erase the boards contents */
 cleanboard();

 /* User is at default location */
 xpos = 0;
 ypos = 0;

 /* you haven't done any move at this moment */
 moves = 0;

 /* Display the cool victory dance */
 displaygame();
 vdance();

 /* Display Middle-'+' then wait for user input and then clear it */
 middleplus();
 getkey();
 middleplus();

 /* load the (first) level */
 loadlevel();

 /* make the current button 'pushed in' */
 pushbutton();

 inloop = 1;
 while(inloop == 1)
   {
   /* Wait for a key and save it in 'key' */
   key = getkey();
   /* only the TI86 has two different buttons to exit.
      On the other calcs kExit & kQuit are just mirrored
      to each other */
  #ifdef TI86
   if(key == kExit || key == kQuit)
  #else
   if(key == kExit)
  #endif
     {
     inloop = 0;
     }
   if(key == kUp && ypos > 0)
     {
     releasebutton();
     ypos--;
     pushbutton();
     }
   if(key == kDown && ypos < 4)
     {
     releasebutton();
     ypos++;
     pushbutton();
     }
   if(key == kRight && xpos < 4)
     {
     releasebutton();
     xpos++;
     pushbutton();
     }
   if(key == kLeft && xpos > 0)
     {
     releasebutton();
     xpos--;
     pushbutton();
     }
   if(key == kEnter || key == kAns)
     {
     invert(xpos,ypos);
     invert(xpos+1,ypos);
     invert(xpos-1,ypos);
     invert(xpos,ypos+1);
     invert(xpos,ypos-1);

     if(moves < 100)        /* have to cap moves somewhere, don't I */
         moves++;

     /* Check for win */

     cmate=0;

     for(x = 0; x < 25; x++)
       cmate += board[x];

     if(cmate==0)
       {
       releasebutton();
       moves -= minmoves;
       if(moves > 10)
         {
         for(a=0; a<8; a++)
           {
           /* Display X, I.E. User failed to pass level in minmoves + 10 */
           /* (size-) optimized this part. I hope I did it right */
           /* ----Henk Poley---- */
           for(b = 0; b < 4; b++)
             {
             invert(b,b);
             invert(b,4-b);
             }
           delay(12);
           }
         /* Reset level info */
         loadlevel();
         }
       else if(moves != 0)
         {
         char ys,xs,p;
         for(a=0; a<8; a++)
           {
           ys = 0;
           xs = 0;
           for(p=0; p<moves; p++)
             {
             invert(xs,ys);
             xs++;
             if(xs > 4)
               {
               xs = 0;
               ys++;
               }
             }
           delay(12);
           }
         level++;
         loadlevel();
         }
       else          /* Score user beat level in least amount of moves */
         {           /* Possible                                       */
         vdance();
         level++;
         loadlevel();
         }
       }
     }
   }

 /*
  * When quiting reset everything so the screen looks "Nice and Pretty"
  * CrASH (TI-82) flushes the screen itself
  */
 #ifndef __TI82__
 clrlcd;
 clrscrn;
 setxy(0,0);
 #endif
 }

loadlevel() {
 char x, y, z, p;
 int  ls;

 /* reset gameboard */
 cleargameboard();

 /* If user has betten final level then give user the final level again */
 if(level == 51)
   {
   level = 50;
   }

 /* Routine to show n load */
 x = level - 1;
 y = 0;

 minmoves = ( ( x ) / 5 ) + 6;

 while(x > 4)
   {
   x-=5;
   y++;
   }
 for(z = 0; z < 5; z++)
   {
   if(level != 50)      /* Content needed already displayed */
     {
     invert(x,y);
     }
   delay(50);
   if(level > 25 && level != 50)
     {
     invert(4,4);
     }
   invert(x,y);
   delay(12);
   }

 if(level > 25)
   {
   invert(4,4);
   }

 ls = level * 5;
 p  = 0;
 while(p < 25)
   {
   board[p]   = (levels[ls] >> 4)    ;
   board[p+1] = (levels[ls] >> 3) & 1;
   board[p+2] = (levels[ls] >> 2) & 1;
   board[p+3] = (levels[ls] >> 1) & 1;
   board[p+4] = (levels[ls]     ) & 1;
   ls++;
   p+=5;
   }

 /* Reset everyting about the game logic */
 moves = 0;
 xpos  = 0;
 ypos  = 0;

 /* RE-DRAW The board */
 drawboard();
 pushbutton();
 }

 /* Show the Victory Dance */
vdance() {
  int x;
  for(x = 0; x < 5; x++)
    {
    invert(0,x);
    delay(10);
    invert(0,x);
    }
  for(x = 1; x < 5; x++)
    {
    invert(x,4);
    delay(10);
    invert(x,4);
    }
  for(x = 3; x >= 0; x--)
    {
    invert(4,x);
    delay(10);
    invert(4,x);
    }
  for(x = 3; x > 0; x--)
    {
    invert(x,0);
    delay(10);
    invert(x,0);
    }
  for(x = 1; x < 4; x++)
    {
    invert(1,x);
    delay(10);
    invert(1,x);
    }

  invert(2,3);
  delay(10);
  invert(2,3);

  invert(3,3);
  delay(10);
  invert(3,3);

  invert(3,2);
  delay(10);
  invert(3,2);

  invert(3,1);
  delay(10);
  invert(3,1);

  invert(2,1);
  delay(10);
  invert(2,1);

  invert(2,2);
  delay(10);
  invert(2,2);
}

drawboard() {
  char z;
  char x, y;
  for(x = 0; x < 5; x++)
    {
    for(y = 0; y < 5; y++)
      {
      if(getboard(x,y) == 1)
        {
        for(z = y * ysize + 4;
            z < y * ysize + ysize - 3;
            z++)
          {
          line(x * xsize + 4,
               z,
               x * xsize + xsize - 4,
               z,
               PT_ON);
          }
        }
      }
    }
  RedrawScreen;
  }
  
/* Invert a button and draw proper data on screen */
invert(x,y) char x, y;
  {
  int z;
  if(x > 4 || x < 0 ||
     y > 4 || y < 0)
    {
    return;
    }
  if(getboard(x,y) == 1)
    {
    setboard(x, y, 0);
    for(z = y * ysize + 4;
        z < y * ysize + ysize - 3;
        z++)
      {
      line(x * xsize + 4,
           z,
           x * xsize + xsize - 4,
           z,
           PT_OFF);
      }
    }
    else
    {
    setboard(x, y, 1);
    for(z = y * ysize + 4;
        z < y * ysize + ysize - 3;
        z++)
      {
      line(x * xsize + 4,
           z,
           x * xsize + xsize - 4,
           z,
           PT_ON);
      }
    }
  RedrawScreen;
  }

/* Reset game board */
cleanboard() {
  char x;
  for(x = 0; x < 25; x++)
    {
    board[x] = 0;
    }
  }

/* Clean the board and screen */
cleargameboard() {
  char x,y;
  for(x = 0; x < 5; x++)
    {
    for(y = 0; y < 5; y++)
      {
      if(board[y*5+x] == 1)
        {
        invert(x,y);
        }
      }
    }
  }

pushbutton() {
  /* Clear old button pushed contents */
  /* (size-)optimized, I saw that this (sub-)routine was the same as
     the one in releasebutton, so I putt'd it in another routine
     ----Henk Poley---- */
  ClearOldButton();

  /* Display new button push contents */
  line(xpos * xsize + 1,
       ypos * ysize,
       xpos * xsize + 1,
       ypos * ysize + ysize,
       PT_ON);
  line(xpos * xsize,
       ypos * ysize + 1,
       xpos * xsize + xsize,
       ypos * ysize + 1,
       PT_ON);
  RedrawScreen;
  }

releasebutton() {
  /* Clear old button pushed contents */
  /* see pushbutton... */
  ClearOldButton();

  /* Display new button push contents */
  line(xpos * xsize,
       ypos * ysize + ysize - 1,
       xpos * xsize + xsize,
       ypos * ysize + ysize - 1,
       PT_ON);
  line(xpos * xsize + xsize - 1,
       ypos * ysize,
       xpos * xsize + xsize - 1,
       ypos * ysize + ysize,
       PT_ON);
  RedrawScreen;
  }

/* Draw all 3d boxes */
displaygameboard()
 {
 char x, y, z;

 for(x = 0; x < 5; x++)
   {
   for(y = 0; y < 5; y++)
     {
     box(x * xsize,
         y * ysize,
         x * xsize + xsize,
         y * ysize + ysize,
         PT_ON);
     box(x * xsize + 2,
         y * ysize + 2,
         x * xsize + xsize - 2,
         y * ysize + ysize - 2,
         PT_ON);
     line(x * xsize,
          y * ysize + ysize - 1,
          x * xsize + xsize,
          y * ysize + ysize - 1,
          PT_ON);
     line(x * xsize + xsize - 1,
          y * ysize,
          x * xsize + xsize - 1,
          y * ysize + ysize,
          PT_ON);
     }
   }
 RedrawScreen;
 }

/* the '+' that is drawn on the board in the intro */
middleplus()
 {
 invert(2,2);
 invert(2,1);
 invert(2,3);
 invert(1,2);
 invert(3,2);
 }

ClearOldButton()
 {
 box(xpos * xsize + 1,
     ypos * ysize + 1,
     xpos * xsize + xsize - 1,
     ypos * ysize + ysize - 1,
     PT_OFF);
 }

#asm
levels:
; Bytes is 5 * 52 = 260
; Blank level for header
  .db   00000000b
  .db   00000000b
  .db   00000000b
  .db   00000000b
  .db   00000000b
; Level 1
  .db   00000000b
  .db   00000000b
  .db   00010101b
  .db   00000000b
  .db   00000000b
; Level 2
  .db   00010101b
  .db   00010101b
  .db   00000000b
  .db   00010101b
  .db   00010101b
; Level 3
  .db   00001010b
  .db   00011011b
  .db   00011011b
  .db   00011011b
  .db   00001010b
; Level 4
  .db   00000000b
  .db   00011011b
  .db   00000000b
  .db   00010001b
  .db   00011011b
; Level 5
  .db   00011110b
  .db   00011101b
  .db   00011101b
  .db   00000011b
  .db   00011011b
; Level 6
  .db   00000000b
  .db   00000000b
  .db   00010101b
  .db   00010101b
  .db   00001110b
; Level 7
  .db   00011110b
  .db   00010001b
  .db   00010001b
  .db   00010001b
  .db   00011110b
; Level 8
  .db   00000000b
  .db   00000100b
  .db   00001010b
  .db   00010101b
  .db   00001010b
; Level 9
  .db   00001010b
  .db   00011111b
  .db   00001110b
  .db   00001011b
  .db   00011100b
; Level 10
  .db   00001110b
  .db   00001110b
  .db   00001110b
  .db   00000000b
  .db   00000000b
; Level 11
  .db   00010101b
  .db   00010101b
  .db   00010101b
  .db   00010101b
  .db   00001110b
; Level 12
  .db   00011111b
  .db   00001010b
  .db   00011011b
  .db   00001110b
  .db   00001010b
; Level 13
  .db   00000010b
  .db   00000101b
  .db   00001010b
  .db   00010100b
  .db   00001000b
; Level 14
  .db   00000000b
  .db   00000000b
  .db   00001000b
  .db   00001000b
  .db   00001000b
; Level 15
  .db   00000000b
  .db   00001000b
  .db   00000000b
  .db   00001000b
  .db   00000000b
; Level 16
  .db   00010000b
  .db   00010000b
  .db   00010000b
  .db   00010000b
  .db   00011111b
; Level 17
  .db   00000000b
  .db   00000000b
  .db   00000100b
  .db   00001110b
  .db   00011111b
; Level 18
  .db   00000100b
  .db   00001010b
  .db   00010101b
  .db   00001010b
  .db   00000100b
; Level 19
  .db   00010101b
  .db   00000000b
  .db   00010101b
  .db   00000000b
  .db   00010101b
; Level 20
  .db   00000000b
  .db   00000000b
  .db   00010001b
  .db   00000000b
  .db   00000000b
; Level 21
  .db   00001111b
  .db   00001000b
  .db   00001110b
  .db   00001000b
  .db   00001000b
; Level 22
  .db   00001110b
  .db   00010001b
  .db   00010001b
  .db   00010001b
  .db   00001110b
; Level 23
  .db   00000000b
  .db   00000000b
  .db   00000111b
  .db   00000110b
  .db   00000100b
; Level 24
  .db   00000000b
  .db   00000000b
  .db   00010001b
  .db   00011111b
  .db   00001001b
; Level 25
  .db   00010000b
  .db   00011000b
  .db   00011100b
  .db   00011110b
  .db   00001111b
; Level 26
  .db   00010001b
  .db   00010001b
  .db   00011111b
  .db   00010001b
  .db   00010001b
; Level 27
  .db   00000100b
  .db   00001110b
  .db   00000100b
  .db   00000100b
  .db   00000100b
; Level 28
  .db   00000000b
  .db   00000000b
  .db   00000111b
  .db   00000111b
  .db   00000111b
; Level 29
  .db   00000000b
  .db   00001000b
  .db   00000000b
  .db   00000000b
  .db   00000000b
; Level 30
  .db   00000000b
  .db   00000000b
  .db   00000100b
  .db   00000000b
  .db   00000000b
; Level 31
  .db   00010001b
  .db   00011001b
  .db   00010101b
  .db   00010011b
  .db   00010001b
; Level 32
  .db   00011111b
  .db   00000010b
  .db   00000100b
  .db   00001000b
  .db   00011111b
; Level 33
  .db   00000010b
  .db   00000010b
  .db   00010101b
  .db   00010001b
  .db   00010011b
; Level 34
  .db   00000101b
  .db   00010001b
  .db   00010001b
  .db   00001101b
  .db   00001111b
; Level 35
  .db   00000011b
  .db   00001010b
  .db   00010001b
  .db   00010101b
  .db   00000000b
; Level 36
  .db   00000100b
  .db   00001010b
  .db   00010001b
  .db   00011111b
  .db   00010001b
; Level 37
  .db   00000000b
  .db   00001110b
  .db   00001110b
  .db   00001110b
  .db   00000000b
; Level 38
  .db   00010101b
  .db   00001010b
  .db   00010101b
  .db   00001010b
  .db   00010101b
; Level 39
  .db   00001010b
  .db   00010000b
  .db   00011000b
  .db   00000110b
  .db   00001010b
; Level 40
  .db   00000000b
  .db   00000000b
  .db   00001010b
  .db   00000000b
  .db   00000000b
; Level 41
  .db   00010001b
  .db   00001010b
  .db   00000100b
  .db   00000100b
  .db   00000100b
; Level 42
  .db   00011100b
  .db   00010010b
  .db   00011100b
  .db   00010010b
  .db   00011100b
; Level 43
  .db   00010001b
  .db   00011010b
  .db   00011100b
  .db   00001000b
  .db   00001110b
; Level 44
  .db   00000000b
  .db   00011011b
  .db   00011111b
  .db   00000100b
  .db   00001110b
; Level 45
  .db   00001110b
  .db   00010100b
  .db   00000111b
  .db   00011110b
  .db   00010101b
; Level 46
  .db   00000100b
  .db   00001110b
  .db   00011111b
  .db   00001110b
  .db   00000100b
; Level 47
  .db   00000100b
  .db   00011111b
  .db   00010100b
  .db   00001001b
  .db   00000001b
; Level 48
  .db   00000000b
  .db   00010001b
  .db   00000100b
  .db   00010001b
  .db   00000000b
; Level 49
  .db   00010001b
  .db   00001010b
  .db   00000100b
  .db   00001010b
  .db   00010001b
; Level 50
  .db   00011111b
  .db   00011111b
  .db   00011111b
  .db   00011111b
  .db   00011111b
; Copy Table
copysize:
   .db 01b
copyterms:
level:
   .db 20h
#endasm
